# -*- coding:utf-8 -*-
#
# Copyright (C) 2012, Maximilian Köhl <linuxmaxi@googlemail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

__version__ = "1.2"
__project__ = "Python Locales"
__short_name__ = "pylocales"
__authors__ = "Maximilian Köhl"
__emails__ = "linuxmaxi@googlemail.com"
__website__ = "http://koehlma.github.com/projects/pygtkspellcheck.html"
__source__ = "https://github.com/koehlma/pygtkspellcheck/"
__vcs__ = "git://github.com/koehlma/pygtkspellcheck.git"
__copyright__ = "2012, Maximilian Köhl"
__desc_short__ = "query the ISO 639/3166 database about a country or a language."
__desc_long__ = (
    "Query the ISO 639/3166 database about a country or a"
    "language. The locales database contains ISO 639 language"
    "definitions and ISO 3166 country definitions. This package"
    "provides translation for country and language names if"
    "the iso-code messages are installed on your system."
)

__metadata__ = {
    "__version__": __version__,
    "__project__": __project__,
    "__short_name__": __short_name__,
    "__authors__": __authors__,
    "__emails__": __emails__,
    "__website__": __website__,
    "__source__": __source__,
    "__vcs__": __vcs__,
    "__copyright__": __copyright__,
    "__desc_short__": __desc_short__,
    "__desc_long__": __desc_long__,
}

from .locales import (
    Country,
    Language,
    LanguageNotFound,
    CountryNotFound,
    code_to_name,
)

__all__ = ["Country", "Language", "LanguageNotFound", "CountryNotFound", "code_to_name"]



# -*- coding:utf-8 -*-
#
# Copyright (C) 2012, Carlos Jenkins <carlos@jenkins.co.cr>
# Copyright (C) 2012-2016, Maximilian Köhl <mail@koehlma.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
This module extracts the .dic and .aff (Hunspell) dictionaries from any given
.oxt extension.

Extensions could be found at:

    http://extensions.services.openoffice.org/dictionary
"""

import functools
import gettext
import logging
import os
import shutil
import sys
import warnings
import xml.dom.minidom
import xml.parsers.expat
import zipfile

# enable deprecation warnings
warnings.simplefilter("always", DeprecationWarning)

# public objects
__all__ = [
    "extract",
    "batch_extract",
    "BadXml",
    "BadExtensionFile",
    "ExtractPathIsNoDirectory",
    "BATCH_SUCCESS",
    "BATCH_ERROR",
    "BATCH_WARNING",
]

# logger
logger = logging.getLogger(__name__)

# translation
locale_name = "py{}gtkspellcheck".format(sys.version_info.major)
_ = gettext.translation(locale_name, fallback=True).gettext


class BadXml(Exception):
    """
    The XML dictionary registry is not valid XML.
    """


class BadExtensionFile(Exception):
    """
    The extension has a wrong file format, should be a ZIP file.
    """


class ExtractPathIsNoDirectory(Exception):
    """
    The given `extract_path` is no directory.
    """


def find_dictionaries(registry):
    def oor_name(name, element):
        return element.attributes["oor:name"].value.lower() == name

    def get_property(name, properties):
        property = list(filter(functools.partial(oor_name, name), properties))
        if property:
            return property[0].getElementsByTagName("value")[0]

    result = []

    # find all "node" elements which have "dictionaries" as "oor:name" attribute
    for dictionaries in filter(
        functools.partial(oor_name, "dictionaries"),
        registry.getElementsByTagName("node"),
    ):
        # for all "node" elements in this dictionary nodes
        for dictionary in dictionaries.getElementsByTagName("node"):
            # get all "prop" elements
            properties = dictionary.getElementsByTagName("prop")
            # get the format property as text
            format = get_property("format", properties).firstChild.data.strip()
            if format and format == "DICT_SPELL":
                # find the locations property
                locations = get_property("locations", properties)
                # if the location property is text:
                # %origin%/dictionary.aff %origin%/dictionary.dic
                if locations.firstChild.nodeType == xml.dom.Node.TEXT_NODE:
                    locations = locations.firstChild.data
                    locations = locations.replace("%origin%/", "").strip()
                    result.append(locations.split())
                # otherwise:
                # <i>%origin%/dictionary.aff</i> <i>%origin%/dictionary.dic</i>
                else:
                    locations = [
                        item.firshChild.data.replace("%origin%/", "").strip()
                        for item in locations.getElementsByTagName("it")
                    ]
                    result.append(locations)

    return result


def extract(filename, target, override=False):
    """
    Extract Hunspell dictionaries out of LibreOffice ``.oxt`` extensions.

    :param filename: path to the ``.oxt`` extension
    :param target: path to extract Hunspell dictionaries to
    :param override: override existing files in the target directory
    :rtype: list of the extracted dictionaries

    This function extracts the Hunspell dictionaries (``.dic`` and ``.aff``
    files) from the given ``.oxt`` extension found to ``target``.

    Extensions could be found at:

        http://extensions.services.openoffice.org/dictionary
    """
    # TODO 5.0: remove this function
    warnings.warn(
        (
            'call to deprecated function "{}", '
            'moved to separate package "oxt_extract", '
            "will be removed in pygtkspellcheck 5.0"
        ).format(extract.__name__),
        category=DeprecationWarning,
    )
    try:
        with zipfile.ZipFile(filename, "r") as extension:
            files = extension.namelist()

            registry = "dictionaries.xcu"
            if registry not in files:
                for filename in files:
                    if filename.lower().endswith(registry):
                        registry = filename

            if registry in files:
                registry = xml.dom.minidom.parse(extension.open(registry))
                dictionaries = find_dictionaries(registry)
                extracted = []
                for dictionary in dictionaries:
                    for filename in dictionary:
                        dict_file = os.path.join(target, os.path.basename(filename))
                        if not os.path.exists(dict_file) or (
                            override and os.path.isfile(dict_file)
                        ):
                            if filename in files:
                                with open(dict_file, "wb") as _target:
                                    with extension.open(filename, "r") as _source:
                                        extracted.append(os.path.basename(filename))
                                        _target.write(_source.read())
                            else:
                                logger.warning(
                                    "dictionary exists in registry "
                                    "but not in the extension zip"
                                )
                        else:
                            logging.warning(
                                (
                                    'dictionary file "{}" already exists '
                                    "and not overriding it"
                                ).format(dict_file)
                            )
                return extracted
    except zipfile.BadZipfile:
        raise BadExtensionFile("extension is not a valid ZIP file")
    except xml.parsers.expat.ExpatError:
        raise BadXml("dictionary registry is not valid XML")


BATCH_SUCCESS = "success"
BATCH_ERROR = "error"
BATCH_WARNING = "warning"


def batch_extract(oxt_path, extract_path, override=False, move_path=None):
    """
    Uncompress, read and install LibreOffice ``.oxt`` dictionaries extensions.

    :param oxt_path: path to a directory containing the ``.oxt`` extensions
    :param extract_path: path to extract Hunspell dictionaries files to
    :param override: override already existing files
    :param move_path: optional path to move the ``.oxt`` files after processing
    :rtype: generator over all extensions, yielding result, extension name,
        error, extracted dictionaries and translated error message - result
        would be :const:`BATCH_SUCCESS` for success, :const:`BATCH_ERROR` if
        some error happened or :const:`BATCH_WARNING` which contain some warning
        messages instead of errors

    This function extracts the Hunspell dictionaries (``.dic`` and ``.aff``
    files) from all the ``.oxt`` extensions found on ``oxt_path`` directory to
    the ``extract_path`` directory.

    Extensions could be found at:

        http://extensions.services.openoffice.org/dictionary

    In detail, this functions does the following:

    1. find all the ``.oxt`` extension files within ``oxt_path``
    2. open (unzip) each extension
    3. find the dictionary definition file within (*dictionaries.xcu*)
    4. parse the dictionary definition file and locate the dictionaries files
    5. uncompress those files to ``extract_path``


    By default file overriding is disabled, set ``override`` parameter to True
    if you want to enable it. As additional option, each processed extension can
    be moved to ``move_path``.

    Example::

        for result, name, error, dictionaries, message in oxt_extract.batch_extract(...):
            if result == oxt_extract.BATCH_SUCCESS:
                print('successfully extracted extension "{}"'.format(name))
            elif result == oxt_extract.BATCH_ERROR:
                print('could not extract extension "{}"'.format(name))
                print(message)
                print('error {}'.format(error))
            elif result == oxt_extract.BATCH_WARNING:
                print('warning during processing extension "{}"'.format(name))
                print(message)
                print(error)

    """

    # TODO 5.0: remove this function
    warnings.warn(
        (
            'call to deprecated function "{}", '
            'moved to separate package "oxt_extract", '
            "will be removed in pygtkspellcheck 5.0"
        ).format(extract.__name__),
        category=DeprecationWarning,
    )

    # get the real, absolute and normalized path
    oxt_path = os.path.normpath(os.path.abspath(os.path.realpath(oxt_path)))

    # check that the input directory exists
    if not os.path.isdir(oxt_path):
        return

    # create extract directory if not exists
    if not os.path.exists(extract_path):
        os.makedirs(extract_path)

    # check that the extract path is a directory
    if not os.path.isdir(extract_path):
        raise ExtractPathIsNoDirectory("extract path is not a valid directory")

    # get all .oxt extension at given path
    oxt_files = [
        extension
        for extension in os.listdir(oxt_path)
        if extension.lower().endswith(".oxt")
    ]

    for extension_name in oxt_files:
        extension_path = os.path.join(oxt_path, extension_name)

        try:
            dictionaries = extract(extension_path, extract_path, override)
            yield BATCH_SUCCESS, extension_name, None, dictionaries, ""
        except BadExtensionFile as error:
            logger.error(
                ('extension "{}" is not a valid ZIP file').format(extension_name)
            )
            yield (
                BATCH_ERROR,
                extension_name,
                error,
                [],
                _('extension "{}" is not a valid ZIP file').format(extension_name),
            )
        except BadXml as error:
            logger.error(
                ('extension "{}" has no valid XML dictionary registry').format(
                    extension_name
                )
            )
            yield (
                BATCH_ERROR,
                extension_name,
                error,
                [],
                _('extension "{}" has no valid XML dictionary registry').format(
                    extension_name
                ),
            )

        # move the extension after processing if user requires it
        if move_path is not None:
            # create move path if it doesn't exists
            if not os.path.exists(move_path):
                os.makedirs(move_path)
            # move to the given path only if it is a directory and target
            # doesn't exists
            if os.path.isdir(move_path):
                if (
                    not os.path.exists(os.path.join(move_path, extension_name))
                    or override
                ):
                    shutil.move(extension_path, move_path)
                else:
                    logger.warning(
                        (
                            "unable to move extension, file with same "
                            "name exists within move_path"
                        )
                    )
                    yield (
                        BATCH_WARNING,
                        extension_name,
                        (
                            "unable to move extension, file with same name "
                            "exists within move_path"
                        ),
                        [],
                        _(
                            "unable to move extension, file with same name "
                            "exists within move_path"
                        ),
                    )
            else:
                logger.warning(
                    ("unable to move extension, move_path is not a " "directory")
                )
                yield (
                    BATCH_WARNING,
                    extension_name,
                    ("unable to move extension, move_path is not a " "directory"),
                    [],
                    _("unable to move extension, move_path is not a " "directory"),
                )



# -*- coding:utf-8 -*-
#
# Copyright (C) 2012, Maximilian Köhl <linuxmaxi@googlemail.com>
# Copyright (C) 2012, Carlos Jenkins <carlos@jenkins.co.cr>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

__version__ = "4.0.5"
__project__ = "Python GTK Spellcheck"
__short_name__ = "pygtkspellcheck"
__authors__ = "The Python GTK Spellcheck Authors"
__emails__ = "linuxmaxi@googlemail.com & carlos@jenkins.co.cr"
__website__ = "http://koehlma.github.com/projects/pygtkspellcheck.html"
__download_url__ = "https://github.com/koehlma/pygtkspellcheck/tarball/master"
__source__ = "https://github.com/koehlma/pygtkspellcheck/"
__vcs__ = "git://github.com/koehlma/pygtkspellcheck.git"
__copyright__ = "2012-2022, The Python GTK Spellcheck Authors"
__desc_short__ = """
A Python spell-checking library for GtkTextViews based on Enchant
"""
__desc_long__ = """
A simple but quite powerful spellchecking library written in pure Python for
Gtk based on Enchant. It supports PyGObject as well as PyGtk for Python 2 and
3 with automatic switching and binding detection. For automatic translation of
the user interface it can use Gedit’s translation files.
"""
__metadata__ = {
    "__version__": __version__,
    "__project__": __project__,
    "__short_name__": __short_name__,
    "__authors__": __authors__,
    "__emails__": __emails__,
    "__website__": __website__,
    "__download_url__": __download_url__,
    "__source__": __source__,
    "__vcs__": __vcs__,
    "__copyright__": __copyright__,
    "__desc_short__": __desc_short__,
    "__desc_long__": __desc_long__,
}

from gtkspellcheck.spellcheck import (
    SpellChecker,
    NoDictionariesFound,
)

__all__ = ["SpellChecker", "NoDictionariesFound"]



# -*- coding:utf-8 -*-
#
# Copyright (C) 2012, Maximilian Köhl <linuxmaxi@googlemail.com>
# Copyright (C) 2012, Carlos Jenkins <carlos@jenkins.co.cr>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Query the ISO 639/3166 database about a country or a language. The locales
database contains ISO 639 language definitions and ISO 3166 country definitions.
This package provides translation for country and language names if the
iso-code messages are installed on your system.
"""

import gettext
import logging
import os
import sqlite3
import sys

# public objects
__all__ = ["Country", "Language", "LanguageNotFound", "CountryNotFound", "code_to_name"]

# translation
_translator_language = gettext.translation("iso_639", fallback=True).gettext
_translator_country = gettext.translation("iso_3166", fallback=True).gettext

# Decides where the database is located. If an application provides an
# os.path.get_module_path monkey patch to determine the path where the module
# is located it uses this. If not it searches in the directory of this source
# code file.
__path__ = None
if hasattr(os.path, "get_module_path"):
    __path__ = os.path.get_module_path(__file__)
    if not os.path.isfile(os.path.join(__path__, "locales.db")):
        __path__ = None
if __path__ is None:
    frozen = getattr(sys, "frozen", None)
    if frozen in ("dll", "console_exe", "windows_exe"):
        __path__ = os.path.abspath(os.path.dirname(sys.executable))
    elif frozen == "macosx_app":
        __path__ = os.path.abspath(os.environ["RESOURCEPATH"])
    elif frozen is True:
        # Handle executables produced by PyInstaller.
        __path__ = sys._MEIPASS
    else:
        __path__ = os.path.abspath(os.path.realpath(os.path.dirname(__file__)))


# loading the database
_database = sqlite3.connect(os.path.join(__path__, "locales.db"))

logger = logging.getLogger(__name__)


class LanguageNotFound(Exception):
    """
    The specified language wasn't found in the database.
    """


class CountryNotFound(Exception):
    """
    The specified country wasn't found in the database.
    """


class Country(object):
    def __init__(self, rowid):
        country = _database.execute(
            "SELECT * FROM countries WHERE rowid == ?", (rowid,)
        ).fetchone()
        self.name = country[0]
        self.official_name = country[1]
        self.alpha_2 = country[2]
        self.alpha_3 = country[3]
        self.numeric = country[4]
        self.translation = _translator_country(self.name)

    @classmethod
    def get_country(cls, code, codec):
        country = _database.execute(
            "SELECT rowid FROM countries WHERE %s == ?" % (codec), (code,)
        ).fetchone()
        if country:
            return cls(country[0])
        raise CountryNotFound("code: %s, codec: %s" % (code, codec))

    @classmethod
    def by_alpha_2(cls, code):
        return Country.get_country(code, "alpha_2")

    @classmethod
    def by_alpha_3(cls, code):
        return Country.get_country(code, "alpha_3")

    @classmethod
    def by_numeric(cls, code):
        return Country.get_country(code, "numeric")


class Language(object):
    def __init__(self, rowid):
        language = _database.execute(
            "SELECT * FROM languages WHERE rowid == ?", (rowid,)
        ).fetchone()
        self.name = language[0]
        self.iso_639_2B = language[1]
        self.iso_639_2T = language[2]
        self.iso_639_1 = language[3]
        self.translation = _translator_language(self.name)

    @classmethod
    def get_language(cls, code, codec):
        language = _database.execute(
            "SELECT rowid FROM languages WHERE %s == ?" % (codec), (code,)
        ).fetchone()
        if language:
            return cls(language[0])
        raise LanguageNotFound("code: %s, codec: %s" % (code, codec))

    @classmethod
    def by_iso_639_2b(cls, code):
        return Language.get_language(code, "iso_639_2B")

    @classmethod
    def by_iso_639_2t(cls, code):
        return Language.get_language(code, "iso_639_2T")

    @classmethod
    def by_iso_639_1(cls, code):
        return Language.get_language(code, "iso_639_1")


def code_to_name(code, separator="_"):
    """
    Get the human readable and translated name of a language based on it's code.

    :param code: the code of the language (e.g. de_DE, en_US)
    :param target: separator used to separate language from country
    :rtype: human readable and translated language name
    """
    logger.debug('requesting name for code "{}"'.format(code))
    code = code.split(separator)
    if len(code) > 1:
        lang = Language.by_iso_639_1(code[0]).translation
        country = Country.by_alpha_2(code[1]).translation
        return "{} ({})".format(lang, country)
    else:
        return Language.by_iso_639_1(code[0]).translation



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['gtkspellcheck', 'gtkspellcheck._pylocales']

package_data = \
{'': ['*']}

install_requires = \
['PyGObject>=3.42.1,<4.0.0', 'pyenchant>=3.0,<4.0']

extras_require = \
{'docs': ['sphinx>=4.5.0,<5.0.0', 'myst-parser>=0.18.0,<0.19.0']}

setup_kwargs = {
    'name': 'pygtkspellcheck',
    'version': '5.0.1',
    'description': 'A simple but quite powerful spellchecking library for GTK written in pure Python.',
    'long_description': '# Python GTK Spellcheck\n\n[![PyPi Project Page](https://img.shields.io/pypi/v/pygtkspellcheck.svg?&label=latest%20version)](https://pypi.python.org/pypi/pygtkspellcheck)\n[![Documentation](https://readthedocs.org/projects/pygtkspellcheck/badge/?version=latest)](https://pygtkspellcheck.readthedocs.org/en/latest/)\n\nPython GTK Spellcheck is a simple but quite powerful spellchecking library for GTK written in pure Python. It\'s spellchecking component is based on [Enchant](http://www.abisource.com/projects/enchant/) and it supports both GTK 3 and 4 via [PyGObject](https://live.gnome.org/PyGObject/).\n\n**⚡️ News:** Thanks to [@cheywood](https://github.com/cheywood), Python GTK Spellcheck now supports GTK 4! 🎉\n\n**🟢 Status:** This project is mature, actively maintained, and open to contributions and co-maintainership.\n\n\n## ✨ Features\n\n- **spellchecking** based on [Enchant](http://www.abisource.com/projects/enchant/) for `GtkTextView`\n- support for word, line, and multiline **ignore regular expressions**\n- support for both **GTK 3 and 4** via [PyGObject](https://live.gnome.org/PyGObject/) for Python 3\n- configurable extra word characters such as `\'`\n- localized names of the available languages based on [ISO-Codes](http://pkg-isocodes.alioth.debian.org/)\n- support for custom ignore tags and hot swap of `GtkTextBuffer`\n- support for Hunspell (LibreOffice) and Aspell (GNU) dictionaries\n\n<p align="center">\n  <img src="https://raw.githubusercontent.com/koehlma/pygtkspellcheck/master/docs/screenshots/screenshot.png" alt="Screenshot" />\n</p>\n\n\n## 🚀 Getting Started\n\nPython GTK Spellcheck is available from the [Python Package Index](https://pypi.python.org/pypi/pygtkspellcheck):\n```sh\npip install pygtkspellcheck\n```\nDepending on your distribution, you may also find Python GTK Spellcheck in your package manager.\nFor instance, on Debian you may want to install the [`python3-gtkspellcheck`](https://packages.debian.org/bullseye/python3-gtkspellcheck) package.\n\n\n## 🥳 Showcase\n\nOver time, several projects have used Python GTK Spellcheck or are still using it. Among those are:\n\n- [Nested Editor](http://nestededitor.sourceforge.net/about.html): “Specialized editor for structured documents.”\n- [Cherry Tree](http://www.giuspen.com/cherrytree/): “A hierarchical note taking application, […].”\n- [Zim](http://zim-wiki.org/): “Zim is a graphical text editor used to maintain a collection of wiki pages.”\n- [REMARKABLE](http://remarkableapp.github.io/): “The best markdown editor for Linux and Windows.”\n- [RedNotebook](http://rednotebook.sourceforge.net/): “RedNotebook is a modern journal.”\n- [Reportbug](https://packages.debian.org/stretch/reportbug): “Reports bugs in the Debian distribution.”\n- [UberWriter](http://uberwriter.wolfvollprecht.de/): “UberWriter is a writing application for markdown.”\n- [Gourmet](https://github.com/thinkle/gourmet): “Gourmet Recipe Manager is a manager, editor, and organizer for recipes.“\n\n\n## 🔖 Versions\n\nVersion numbers follow [Semantic Versioning](http://semver.org/). However, the update from 3 to 4 pertains only API incompatible changes in `oxt_extract` and not the spellchecking component. The update from 4 to 5 removed support for Python 2, GTK 2, `pylocales`, and the `oxt_extract` API. Otherwise, the API is still compatible with version 3.\n\n\n## 📚 Documentation\n\nThe documentation is available at [Read the Docs](http://pygtkspellcheck.readthedocs.org/).\n\n\n## 🏗 Contributing\n\nWe welcome all kinds of contributions! ❤️\n\nFor minor changes and bug fixes feel free to simply open a pull request. For major changes impacting the overall design of Python GTK Spellcheck, please first [start a discussion](https://github.com/koehlma/pygtkspellcheck/discussions/new?category=ideas) outlining your idea.\n\nBy submitting a PR, you agree to license your contributions under “GPLv3 or later”.\n',
    'author': 'Maximilian Köhl',
    'author_email': 'mail@koehlma.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/koehlma/pygtkspellcheck',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
